from dataclasses import asdict, dataclass
from pathlib import Path
from typing import List

import toml


class InvalidConfigException(Exception):
    pass


@dataclass
class MutableConfig:
    exclude: List[str] = None
    application_import_names: List[str] = None

    def dict(self):
        return asdict(self)


@dataclass
class Config(MutableConfig):
    line_length: int = 88


def load_config() -> Config:
    config_path = Path.cwd() / 'pyproject.toml'
    if not config_path.exists():
        return Config()

    try:
        project_config = toml.load(config_path)
    except Exception as exc:
        raise InvalidConfigException(f'Invalid config: {exc}') from exc

    tool_config = project_config.get('tool', {}).get('vgs-style', {})
    try:
        user_config = MutableConfig(**tool_config)
    except TypeError as exc:
        raise InvalidConfigException(str(exc)) from exc

    return Config(**user_config.dict())



from abc import ABC, abstractmethod, abstractproperty
from pathlib import Path
from typing import List

from ..config import Config


class Linter(ABC):
    def __init__(self, config: Config) -> None:
        pass

    @abstractmethod
    def run(self, paths: List[Path]) -> List[str]:
        pass

    @abstractproperty
    def dump_config(self, dst: Path) -> None:
        pass

    @abstractproperty
    def name(self) -> str:
        pass



from pathlib import Path
from typing import List

from flake8.api import legacy
from flake8.formatting.base import BaseFormatter
from flake8.main.application import Application
from flake8.options import config as flake_config
from flake8.style_guide import Violation

from .base import Linter
from ..config import Config


class Formatter(BaseFormatter):
    errors: List[str] = None

    def after_init(self):
        Formatter.errors = []

    def format(self, error: Violation) -> str:
        return (
            f'{error.filename}:{error.line_number}:{error.column_number}'
            f': {error.code} {error.text}'
        )

    def write(self, line: str, source: str):
        Formatter.errors.append(line)


class Flake8Linter(Linter):
    def __init__(self, config: Config) -> None:
        super().__init__(config)

        self.app = Application()
        args = [
            '--isolated',
            '--max-line-length',
            str(config.line_length),
            '--import-order-style',
            'pep8',
        ]
        if config.exclude:
            args.extend(['--extend-exclude', config.exclude])
        if config.application_import_names:
            args.extend(['--application-import-names', config.application_import_names])

        prelim_opts, remaining_args = self.app.parse_preliminary_options(args)

        config_finder = flake_config.ConfigFileFinder(
            self.app.program,
            prelim_opts.append_config,
            config_file=prelim_opts.config,
            ignore_config_files=prelim_opts.isolated,
        )
        self.app.find_plugins(config_finder)
        self.app.register_plugin_options()
        self.app.parse_configuration_and_cli(config_finder, remaining_args)
        self.app.make_formatter()
        self.app.make_guide()
        self.app.make_file_checker_manager()

    def run(self, paths: List[Path]) -> List[str]:
        style_guide = legacy.StyleGuide(self.app)
        style_guide.init_report(Formatter)
        style_guide.check_files(paths)
        return Formatter.errors

    def dump_config(self, dst: Path) -> None:
        with open(dst / '.flake8', 'w') as out:
            out.write('[flake8]\n')
            out.write(f'max-line-length = {self.app.options.max_line_length}\n')
            out.write(f'import-order-style = pep8\n')
            if self.app.options.extend_exclude:
                out.write(
                    f'extra-exclude = {",".join(self.app.options.extend_exclude)}\n'
                )
            if self.app.options.application_import_names:
                out.write(
                    'application-import-names = '
                    f'{",".join(self.app.options.application_import_names)}\n'
                )

    @property
    def name(self):
        return 'flake8'



from dataclasses import dataclass, field
from pathlib import Path
from typing import List

import black

from .base import Linter
from ..config import Config


class BlackLinter(Linter):
    def __init__(self, config: Config) -> None:
        self.config = {
            'line_length': config.line_length,
            'skip_string_normalization': True,
        }
        if config.exclude:
            self.config['exclude'] = '|'.join(
                expr.replace('.', r'\.').replace('*', '.*')
                for expr in config.exclude.split(',')
            )

    def dump_config(self, dst: Path) -> None:
        with open(dst / 'black.toml', 'w') as out:
            out.write('[tool.black]\n')
            out.write(f'line_length = {self.config["line_length"]}\n')
            out.write(
                'skip-string-normalization = '
                f'{str(self.config["skip_string_normalization"]).lower()}\n'
            )
            if 'exclude' in self.config:
                out.write(f'exclude = "{self.config["exclude"]}"\n')

    def run(self, paths: List[Path]) -> List[str]:
        return self._run(paths, check=True)

    def format(self, paths: List[Path]) -> None:
        self._run(paths, check=False)

    @property
    def name(self):
        return 'black'

    def _run(self, paths: List[Path], check: bool) -> List[str]:
        # Hardcoded params
        quiet = True
        verbose = False
        force_exclude = None
        fast = True
        diff = False
        color = False
        include = black.DEFAULT_INCLUDES

        write_back = black.WriteBack.from_configuration(
            check=check, diff=diff, color=color
        )
        versions = set()
        mode = black.Mode(
            target_versions=versions,
            line_length=self.config['line_length'],
            is_pyi=False,
            string_normalization=not self.config['skip_string_normalization'],
            experimental_string_processing=False,
        )
        report = Report(check=check)

        sources = black.get_sources(
            ctx=None,  # TODO: emulate click context
            src=paths,
            quiet=quiet,
            verbose=verbose,
            include=include,
            exclude=self.config.get('exclude', black.DEFAULT_EXCLUDES),
            force_exclude=force_exclude,
            report=report,
        )

        if not sources:
            return []

        if len(sources) == 1:
            black.reformat_one(
                src=sources.pop(),
                fast=fast,
                write_back=write_back,
                mode=mode,
                report=report,
            )
        else:
            black.reformat_many(
                sources=sources,
                fast=fast,
                write_back=write_back,
                mode=mode,
                report=report,
            )

        return report.errors


@dataclass
class Report(black.Report):
    errors: List[str] = field(default_factory=list)

    def done(self, src: Path, changed: black.Changed) -> None:
        if changed is black.Changed.YES and self.check:
            self.errors.append(f'{src}: would reformat')
            self.change_count += 1

    def failed(self, src: Path, message: str) -> None:
        self.errors.append(f'{src}: cannot format')
        self.failure_count += 1



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['vgsstyle', 'vgsstyle.linters']

package_data = \
{'': ['*']}

install_requires = \
['black>=20.8b1,<21.0',
 'click>=7.1.2,<8.0.0',
 'flake8-import-order>=0.18.1,<0.19.0',
 'flake8>=3.8.4,<4.0.0',
 'toml>=0.10.2,<0.11.0']

entry_points = \
{'console_scripts': ['vgs-style = vgsstyle.cli:main']}

setup_kwargs = {
    'name': 'vgs-style',
    'version': '0.2.2',
    'description': 'Library for linting and formatting Python source code',
    'long_description': None,
    'author': 'Petro Melnykov',
    'author_email': 'petro.melnykov@vgs.io',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



import sys
from pathlib import Path
from typing import List

import click

from .config import InvalidConfigException, load_config
from .linters.base import Linter
from .linters.black import BlackLinter
from .linters.flake import Flake8Linter


LINTERS: List[Linter] = [Flake8Linter, BlackLinter]


@click.group()
@click.pass_context
def main(ctx: click.Context):
    ctx.ensure_object(dict)
    try:
        ctx.obj["config"] = load_config()
    except InvalidConfigException as exc:
        click.echo(f"Invalid config: {exc}", err=True)
        ctx.exit(1)


@main.command()
@click.pass_context
@click.argument("paths", nargs=-1, type=click.Path(exists=True))
def lint(ctx: click.Context, paths: List[str] = None) -> None:
    ok = True

    if not paths:
        paths = [Path.cwd()]

    for linter_cls in LINTERS:
        linter = linter_cls(ctx.obj["config"])
        for error in linter.run(paths):
            if ok:
                ok = False
            click.echo(f"[{linter.name}] {error}", err=True)

    if not ok:
        sys.exit(1)


@main.command()
@click.pass_context
@click.argument("paths", nargs=-1, type=click.Path(exists=True))
def format(ctx: click.Context, paths: List[str] = None) -> None:
    black = BlackLinter(config=ctx.obj["config"])
    black.format(paths)


@main.command()
@click.pass_context
@click.argument("dst", type=click.Path(exists=True, file_okay=False, writable=True))
def dump_configs(ctx: click.Context, dst: str) -> None:
    dst = Path(dst)
    for linter_cls in LINTERS:
        linter_cls(ctx.obj["config"]).dump_config(dst)


if __name__ == "__main__":
    main()
